/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific
   main-programmer: Patrick Boenzli
   co-programmer: Benjamin Grauer

*/
#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_WEAPON

#include "test_bullet.h"

#include "fast_factory.h"

#include "state.h"
#include "list.h"
#include "class_list.h"

#include "particle_engine.h"
#include "particle_emitter.h"
#include "particle_system.h"


using namespace std;

CREATE_FAST_FACTORY_STATIC(TestBullet, CL_TEST_BULLET);

/**
 *  standard constructor
*/
TestBullet::TestBullet () : Projectile()
{
  this->setClassID(CL_TEST_BULLET, "TestBullet");

  float modelSize = .3;
  this->loadModelWithScale("models/projectiles/orx-rocket.obj", .3);

  this->energyMin = 1;
  this->energyMax = 10;
  this->remove();
  this->lifeSpan = 5;

  this->emitter = new ParticleEmitter(Vector(0,1,0), M_2_PI, 100, 5);
  this->emitter->setParent(this);
  this->emitter->setSpread(M_2_PI);
}


/**
 *  standard deconstructor
*/
TestBullet::~TestBullet ()
{
  // delete this->emitter;

  /* this is normaly done by World.cc by deleting the ParticleEngine */
  if (TestBullet::trailParticles != NULL && ClassList::getList(CL_TEST_BULLET)->getSize() <= 1)
  {
    if (ClassList::exists(TestBullet::trailParticles, CL_PARTICLE_SYSTEM))
      delete TestBullet::trailParticles;
    TestBullet::trailParticles = NULL;
  }
  if (TestBullet::explosionParticles != NULL && ClassList::getList(CL_TEST_BULLET)->getSize() <= 1)
  {
    if (ClassList::exists(TestBullet::explosionParticles, CL_PARTICLE_SYSTEM))
      delete TestBullet::explosionParticles;
    TestBullet::explosionParticles = NULL;
  }

}

ParticleSystem* TestBullet::trailParticles = NULL;
ParticleSystem* TestBullet::explosionParticles = NULL;

void TestBullet::activate()
{
  State::getWorldEntityList()->add(this);
  if (unlikely(TestBullet::trailParticles == NULL))
  {
    TestBullet::trailParticles = new ParticleSystem(1000, PARTICLE_SPRITE);
    TestBullet::trailParticles->setName("TestBulletTrailParticles");
    TestBullet::trailParticles->setLifeSpan(.5, .3);
    TestBullet::trailParticles->setRadius(0.0, .5);
    TestBullet::trailParticles->setRadius(0.5, 2.0);
    TestBullet::trailParticles->setRadius(1.0, 5.0);
    TestBullet::trailParticles->setColor(0.0, 1,0,0,.7);
    TestBullet::trailParticles->setColor(0.5, .8,.8,0,.5);
    TestBullet::trailParticles->setColor(1.0, .7,.7,.7,.0);
  }
  if (unlikely(TestBullet::explosionParticles == NULL))
  {
    TestBullet::explosionParticles = new ParticleSystem(1000, PARTICLE_SPRITE);
    TestBullet::explosionParticles->setName("TestBulletExplosionParticles");
    TestBullet::explosionParticles->setLifeSpan(.5, .3);
    TestBullet::explosionParticles->setRadius(0.0, 10);
    TestBullet::explosionParticles->setRadius(.5, 20.0);
    TestBullet::explosionParticles->setRadius(1.0, 3.0);
    TestBullet::explosionParticles->setColor(0.0, 0,1,0,.7);
    TestBullet::explosionParticles->setColor(0.5, .8,.8,0,.5);
    TestBullet::explosionParticles->setColor(1.0, 1,1,1,.0);
  }

  ParticleEngine::getInstance()->addConnection(this->emitter, TestBullet::trailParticles);

  this->emitter->setEmissionRate(20.0);
  this->emitter->setEmissionVelocity(10.0);
}


void TestBullet::deactivate()
{
  ParticleEngine::getInstance()->breakConnections(this->emitter);
  this->lifeCycle = 0.0;

//  GarbageCollector::getInstance()->collect(this);
  State::getWorldEntityList()->remove(this);
  TestBullet::fastFactory->kill(this);
}


void TestBullet::collidesWith(WorldEntity* entity, const Vector& location)
{
  if (this->hitEntity != entity && entity->isA(CL_NPC))
    this->destroy();
  this->hitEntity = entity;
}

/**
 *  signal tick, time dependent things will be handled here
 * @param time since last tick
*/
void TestBullet::tick (float time)
{
  //Vector v = *this->flightDirection * ( this->speed * time * 1000 + 0.1);
  Vector v = this->velocity * (time);
  this->shiftCoor(v);

  this->lifeCycle += time/this->lifeSpan;
  if( this->lifeCycle >= 1.0)
    {
      PRINTF(5)("FINALIZE==========================\n");
      PRINTF(5)("current life cycle is: %f\n", this->lifeCycle);
      PRINTF(5)("FINALIZE===========================\n");

      this->deactivate();
    }
}

/**
 *  the function gets called, when the projectile is destroyed
*/
void TestBullet::destroy ()
{
  printf("DESTROY TestBullet\n");
  this->lifeCycle = .95;
  ParticleEngine::getInstance()->breakConnection(this->emitter, TestBullet::trailParticles);
  ParticleEngine::getInstance()->addConnection(this->emitter, TestBullet::explosionParticles);

  this->emitter->setEmissionRate(1000.0);
  this->emitter->setEmissionVelocity(50.0);
//  this->deactivate();

}


void TestBullet::draw ()
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  float matrix[4][4];
  glTranslatef (this->getAbsCoor ().x, this->getAbsCoor ().y, this->getAbsCoor ().z);
  this->getAbsDir().matrix (matrix);
  glMultMatrixf((float*)matrix);
  glScalef(2.0, 2.0, 2.0);
  this->model->draw();

  glPopMatrix();
}

